#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "suart.h"

#define ADC1_PB2 1


#define KENN_VALNUM 5
uint16_t raKenn_adc[KENN_VALNUM] = {0, 20, 40, 100, 255};
uint16_t raKenn_tmp[KENN_VALNUM] = {1, 20, 40, 100, 255};

uint16_t GetPwm(uint16_t val) {
	uint8_t i;
	for(i = 1; i < KENN_VALNUM; i++) {
		if(raKenn_adc[i] >= val) {
			break;
		}
	}

	i--;
	uint16_t adc_d = raKenn_adc[i+1] - raKenn_adc[i];
	uint16_t tmp_d = raKenn_tmp[i+1] - raKenn_tmp[i];
	tmp_d *= 255;
	val -= raKenn_adc[i];
	return ((tmp_d/adc_d)*val)/255 + raKenn_tmp[i];
}

volatile uint16_t adcval;

ISR(ADC_vect) {  
	adcval = ADCH;
}


int main(void) {
	
	//PB0 = LED-Driver
	//PB3 = UART Tx
	DDRB = (1<<PB0) | (1<<PB3);
	
	//Init Timer for PWM
	TCCR0A = (1<<COM0A1) | (0<<COM0A0) | (0<<COM0B1) | (0<<COM0B0) | (1<<WGM01) | (1<<WGM00);
	//                      ^ 1 for inverted (COM0A)
	TCCR0B = (0<<FOC0A) | (0<<FOC0B) | (0<<WGM02) | (0<<CS02) | (0<<CS01) | (1<<CS00);
	
	//init ADC, Ref = Vcc, free running mode on PB2
	ADMUX = (ADC1_PB2) | (1<<ADLAR);
	ADCSRB = 0; //not really needed
	ADCSRA = (1<<ADEN) | (1<<ADIE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0) | (1<<ADATE) | (1<<ADSC);

	sei();
	
	//buffer for averaging
	#define ADCVAL_CNT 10
	uint8_t adcpos = 0;
	uint8_t adcvals[ADCVAL_CNT];
	
	
	uint8_t brightness = 0;
	
    while(1) {

		//invert adcval for brightness control
		adcvals[adcpos] = 255-adcval;
		
		//do some averaging
		adcpos++;
		if(adcpos >= ADCVAL_CNT) adcpos = 0;
		
		uint16_t avg = 0;
		for(uint8_t i = 0; i < ADCVAL_CNT; i++) {
			avg += adcvals[i];
		}
		avg /= ADCVAL_CNT;
		
		//adjust brightness
		if(brightness < avg) {
			brightness++;
		} else if(brightness > avg) {
			brightness--;
		}
		
		//set brightness
		OCR0A = GetPwm(brightness);
		
		//debug
		printnum(avg);
		Debug_c('\t');
		printnum(brightness);
		
		Debug_c('\t');
		printnum(OCR0A);
		Debug_c('\n');
		
		//wait
		_delay_ms(50);
    }
}